#include "poweroff.h"
#include <winnetwk.h>

void RunProgram(HWND hWnd,PowerSettings *ps)
{
	PROCESS_INFORMATION p;
  STARTUPINFO si;
  DWORD exit_code;
  char *directory=NULL;

  if (ps->program.directory[0]!='\0')
    directory=ps->program.directory;

	si.cb=sizeof(STARTUPINFO);
	si.lpReserved=NULL;
	si.lpDesktop=NULL;
	si.lpTitle=NULL;
	si.dwFlags=0;
	si.cbReserved2=0;
	si.lpReserved2=NULL;
  if (CreateProcess(NULL,ps->program.program,NULL,NULL,FALSE,0,NULL,directory,&si,&p)==0)
  {
    DisplayLastError(ps,hWnd);
    return;
  }
	do
	{
		GetExitCodeProcess(p.hProcess,&exit_code);
	  Sleep(100);
  } while (exit_code==STILL_ACTIVE);
}

char *GetModeStr(PowerSettings *ps)
{
  static char str[100];

  str[0]='\0';
  if (ps->options.force)
    strcpy(str,"FORCE ");
  switch (ps->action)
	{
	  case LOGOFF:
      strcat(str,"LOGOFF");
      break;
	  case REBOOT:
      strcat(str,"REBOOT");
      break;
	  case SHUTDOWN:
      strcat(str,"SHUTDOWN");
      break;
	  case POWEROFF:
      strcat(str,"POWEROFF");
      break;
	  case STANDBY:
      strcat(str,"STANDBY");
      break;
	  case HIBERNATE:
      strcat(str,"HIBERNATE");
      break;
	  case LOCK:
      strcat(str,"LOCK");
      break;
	  case WAKE_ON_LAN:
      strcat(str,"WAKE_ON_LAN");
      break;
		default:
      strcat(str,"UNKNOWN");
	}
  return str;
}

int AquirePrivileges(HWND hWnd,PowerSettings *ps)
{
  HANDLE current_thread,token;
  TOKEN_PRIVILEGES privs;
  LUID luid;

  current_thread=GetCurrentProcess();
  if (!OpenProcessToken(current_thread,TOKEN_ADJUST_PRIVILEGES,&token))
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  if (!LookupPrivilegeValue(NULL,SE_SHUTDOWN_NAME,&luid))
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  privs.PrivilegeCount=1;
  privs.Privileges[0].Attributes=SE_PRIVILEGE_ENABLED;
  privs.Privileges[0].Luid=luid;
  if (!AdjustTokenPrivileges(token,FALSE,&privs,0,NULL,NULL))
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  return 0;
}

int DoExitWindowsEx(HWND hWnd,PowerSettings *ps)
{
  UINT mode;

  switch (ps->action)
  {
    case LOGOFF:
      mode=EWX_LOGOFF;
      break;
    case REBOOT:
      mode=EWX_REBOOT;
      break;
    case SHUTDOWN:
      mode=EWX_SHUTDOWN;
      break;
    case POWEROFF:
      if (ps->windowsversion.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS)
        mode=EWX_SHUTDOWN;
      else
        mode=EWX_POWEROFF;
      break;
  }
  if (ps->options.force)
    mode|=EWX_FORCE;

  if (ps->options.warning)
  {
    if (!ShowWarningMessage(hWnd,ps))
      return 1;
  }

  if (ps->options.run_program)
  {
    RunProgram(hWnd,ps);
  }
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Initiating %s",GetModeStr(ps));

  if (ps->simulate)
    Error(ps,hWnd,"ExitWindowsEx");
  else if (!ExitWindowsEx(mode,0))
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  else
    WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"ExitWindowsEx executed succesfully!");
  return 0;
}

int DoSetSystemPowerState(HWND hWnd,PowerSettings *ps)
{
  if (ps->options.warning)
  {
    if (!ShowWarningMessage(hWnd,ps))
      return 1;
  }

  if (ps->options.run_program)
  {
    RunProgram(hWnd,ps);
  }
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Initiating %s",GetModeStr(ps));

  if (ps->simulate)
    Error(ps,hWnd,"SetSystemPowerState");
  else if (!SetSystemPowerState((ps->action==STANDBY)?TRUE:FALSE,ps->options.force?TRUE:FALSE))
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  else
    WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"SetSystemPowerState executed succesfully!");
  return 0;
}

int DoLockWorkstation(HWND hWnd,PowerSettings *ps)
{
  HINSTANCE lInst;
  FARPROC MyLockWorkstation;

  if (ps->options.warning)
  {
    if (!ShowWarningMessage(hWnd,ps))
      return 1;
  }

  if (ps->options.run_program)
  {
    RunProgram(hWnd,ps);
  }
  if ((lInst=LoadLibrary("user32.dll"))==NULL)
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  MyLockWorkstation=GetProcAddress(lInst,"LockWorkStation");
  if (MyLockWorkstation==NULL)
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Initiating %s",GetModeStr(ps));
  if (ps->simulate)
    Error(ps,hWnd,"MyLockWorkstation");
  else if (!MyLockWorkstation())
  {
    DisplayLastError(ps,hWnd);
    return -1;
  }
  else
    WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"LockWorkStation executed succesfully!");
  FreeLibrary(lInst);
  return 0;
}

int DoInitiateSystemShutdown(HWND hWnd,PowerSettings *ps)
{
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Initiating %s on remote computer %s",GetModeStr(ps),ps->remote.computer_name);
  if (ps->simulate)
    Error(ps,hWnd,"InitiateSystemShutdown");
  else
  {
    NETRESOURCE net;
    char str[MAX_PATH];

    if (!ps->remote.current_credentials)
    {
      if (ps->remote.computer_name[0]!='\\')
        sprintf(str,"\\\\%s\\admin$",ps->remote.computer_name);
      else
        sprintf(str,"%s\\admin$",ps->remote.computer_name);

      net.dwScope=RESOURCE_GLOBALNET;
      net.dwType=RESOURCETYPE_ANY;
      net.dwDisplayType=RESOURCEDISPLAYTYPE_GENERIC;
      net.dwUsage=RESOURCEUSAGE_CONNECTABLE;
      net.lpLocalName=NULL;
      net.lpRemoteName=str;
      net.lpComment=NULL;
      net.lpProvider=NULL;
      if (WNetAddConnection2(&net,ps->remote.password,ps->remote.username,0)!=NO_ERROR)
      {
        DisplayLastError(ps,hWnd);
        return -1;
      }
    }
    if (!InitiateSystemShutdown(ps->remote.computer_name,ps->warning.message,ps->warning.seconds,ps->options.force,(ps->action==REBOOT)?1:0))
    {
      DisplayLastError(ps,hWnd);
      return -1;
    }
    if (!ps->remote.current_credentials)
    {
      WNetCancelConnection2(str,0,FALSE);
    }
  }
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Remote computer has accepted the request");
  if (ps->interactive)
  {
    DisplayInformation(ps,hWnd,"Remote computer has accepted the request");
  }
  return 0;
}

int DoWOL(HWND hWnd,PowerSettings *ps)
{
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Waking up remote computer %s",ps->remote.computer_name);
  if (ps->simulate)
    Error(ps,hWnd,"WakeOnLan");
  else if (WakeOnLan(ps,ps->remote.ip_address,ps->remote.subnet_mask,ps->remote.mac_address))
  {
    Error(ps,hWnd,"WakeOnLAN failed");
    return -1;
  }
  WriteEvent(ps,EVENTLOG_INFORMATION_TYPE,"Wake-On-LAN packet has been sent");
  if (ps->interactive)
  {
    DisplayInformation(ps,hWnd,"Wake-On-LAN packet has been sent");
  }
  return 0;
}

int PowerOff(HWND hWnd,PowerSettings *ps)
{
  int ret;

  ps->schedule.wait=60;
  if (ps->who==REMOTE_COMPUTER && !ps->remote.use_nt && ps->action!=WAKE_ON_LAN)
  {
    char str[500];

    if ((ret=SendRemoteCommand(hWnd,ps,str))!=0)
    {
      if (ret==-2)
        Error(ps,hWnd,str);
      return -1;
    }
    else
      return 0;
  }
  if (ps->windowsversion.dwPlatformId==VER_PLATFORM_WIN32_NT)
  {
    if (AquirePrivileges(hWnd,ps)!=0)
    {
      return -1;
    }
  }
  switch (ps->action)
  {
    case LOGOFF:
    case REBOOT:
    case SHUTDOWN:
    case POWEROFF:
      if (ps->who==LOCAL_COMPUTER)
      {
        ret=DoExitWindowsEx(hWnd,ps);
      }
      else
        ret=DoInitiateSystemShutdown(hWnd,ps);
      break;
    case STANDBY:    
    case HIBERNATE:
      ret=DoSetSystemPowerState(hWnd,ps);
      break;
    case LOCK:
      ret=DoLockWorkstation(hWnd,ps);
      break;
    case WAKE_ON_LAN:
      ret=DoWOL(hWnd,ps);
      break;
  }
  return ret;
}

void DoQuickAction(PowerSettings *ps,Action action)
{
  char actionstr[100];
  Action tmp;

  tmp=ps->action;
  ps->action=action;
  sprintf(actionstr,"Are you sure that you want to %s your pc now?",GetModeStr(ps));
  if (MessageBox(NULL,actionstr,"Are you sure?",MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1)==IDYES)
  {
    ps->when=IMMEDIATE;
    ps->who=LOCAL_COMPUTER;
    ps->options.warning=0;
    ps->options.run_program=0;
    PowerOff(NULL,ps);
  }
  else
    ps->action=tmp;
}

int DoIt(HWND hWnd,PowerSettings *ps)
{
  char str[500];

  if (!SanityCheck(hWnd,ps,str))
  {
    if (ps->who==REMOTE_COMPUTER && ps->remote.use_nt==0 && (ps->when==IMMEDIATE || ps->remote.schedule_remote) && ps->action!=WAKE_ON_LAN)
    {
      if (SendRemoteCommand(hWnd,ps,str)==-2)
      {
        Error(ps,hWnd,str);
        return -1;
      }
    }
    else if (ps->when==IMMEDIATE)
      PowerOff(hWnd,ps);
    else if (ps->when==SCHEDULED)
    {
      ps->remaining_seconds=ps->schedule.seconds;
      ps->active_timer=SetTimer(hWnd,ID_SCHEDULE_TIMER,1000,NULL);
      MinimizePoweroff(hWnd,ps);
    }
    else if (ps->when==PROCESS)
    {
      ps->active_timer=SetTimer(hWnd,ID_PROCESS_TIMER,1000,NULL);
      MinimizePoweroff(hWnd,ps);
    }
  }
  else
  {
    Error(ps,hWnd,str);
    return -2;
  }
  return 0;
}